/****************************************************************************************
*  YAROCK                                                                               *
*  Copyright (c) 2010-2013 Sebastien amardeilh <sebastien.amardeilh+yarock@gmail.com>   *
*                                                                                       *
*  This program is free software; you can redistribute it and/or modify it under        *
*  the terms of the GNU General Public License as published by the Free Software        *
*  Foundation; either version 2 of the License, or (at your option) any later           *
*  version.                                                                             *
*                                                                                       *
*  This program is distributed in the hope that it will be useful, but WITHOUT ANY      *
*  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A      *
*  PARTICULAR PURPOSE. See the GNU General Public License for more details.             *
*                                                                                       *
*  You should have received a copy of the GNU General Public License along with         *
*  this program.  If not, see <http://www.gnu.org/licenses/>.                           *
*****************************************************************************************/
#include "covertask.h"

#include "covers/coverprovider_lastfm.h"
#include "covers/coverprovider_discogs.h"

#include "core/database/database.h"
#include "core/mediaitem/mediaitem.h"
#include "utilities.h"
#include "debug.h"

#include <QSqlField>
#include <QSqlQuery>
#include <QSqlDriver>
#include <QVariant>

#include <QFile>
#include <QImage>


/*
********************************************************************************
*                                                                              *
*    Class CoverTask                                                           *
*                                                                              *
********************************************************************************
*/

CoverTask::CoverTask(QObject *parent) : QObject(parent)
{
    setObjectName("covertask");

    //! init
    m_current_request_id  = 0;
    m_isFullDbSearch      = true;
    m_exit                = false;
    m_isRunning           = false;
}

CoverTask::~CoverTask()
{
}


//! -- CoverTask::setRequest -------------------------------------------------
void CoverTask::setRequest(const QString& artist, const QString& album)
{
    m_map_params.clear();

    RequestParam param = RequestParam();
    param.artist       = artist;
    param.album        = album;
    param.cover_path   = MEDIA::coverName(artist, album);

    const QString path = UTIL::CONFIGDIR + "/albums/" + param.cover_path;

    if(!QFile::exists(path)) {
      m_map_params[1] = param;
    }
    m_isFullDbSearch = false;
}


//! -- CoverTask::run --------------------------------------------------------
void CoverTask::start()
{
    Debug::debug() << "       [CoverTask] run";
    if(m_isFullDbSearch)
      slot_check_database();
    /*else
      -> take request as parameter  */

    m_isRunning = true;
    m_max = m_map_params.size();
    m_current_request_id =1;

    slot_process_cover_search();

    //! Start thread event loop --> makes signals and slots work
    QEventLoop eventLoop;
    eventLoop.exec();
    
    m_isFullDbSearch      = true;
    m_map_params.clear();
}

//! -- slot_finish_cover_search ------------------------------------------------
void CoverTask::slot_finish_cover_search()
{
    Debug::debug() << "       [CoverTask] finished";

    //! deletes previous providers
    qDeleteAll(m_coverprovider_list);
    m_coverprovider_list.clear();

    m_isRunning = false;
    emit finished();
}

//! -- slot_process_cover_search -----------------------------------------------
void CoverTask::slot_process_cover_search()
{
    Debug::debug() << "       [CoverTask] slot_process_cover_search m_current_request_id:" << m_current_request_id;
    if(!m_map_params.isEmpty() && !m_exit)
    {

      if(m_map_params.keys().contains(m_current_request_id)) 
      {
          m_current_param = m_map_params.take(m_current_request_id);

          emit progress ( ((m_max - m_map_params.size())*100)/m_max );

          qDeleteAll(m_coverprovider_list);
          m_coverprovider_list.clear();

          m_coverprovider_list.append( new CoverProvider_LastFm(0) );
          m_coverprovider_list.append( new CoverProvider_Discogs(0) );

          foreach (CoverProvider* provider, m_coverprovider_list) {
            connect(provider, SIGNAL(search_finished(int)), this, SLOT(slot_cover_provider_search_finished()), Qt::QueuedConnection);
            connect(provider, SIGNAL(data_received(int,QByteArray)), this, SLOT(slot_save_cover(int,QByteArray)), Qt::QueuedConnection);
          }

          //! start cover provider
          m_current_provider_id = 0;
          slot_start_cover_provider();
      }
    }
    else {
      slot_finish_cover_search();
    }
}


//! -- slot_start_cover_provider -----------------------------------------------
void CoverTask::slot_start_cover_provider()
{
    if(m_current_provider_id < m_coverprovider_list.size() )
    {
      //Debug::debug() << "       [CoverTask] : slot_start_cover_provider :" << m_current_param.artist;
      //Debug::debug() << "       [CoverTask] : slot_start_cover_provider :" << m_current_param.album;
      CoverProvider* provider = m_coverprovider_list.at(m_current_provider_id);

      Debug::debug() << "       [CoverTask] slot_start_cover_provider  m_current_request_id:" << m_current_request_id;
      provider->set_search(m_current_param.artist, m_current_param.album);
      provider->start_search(m_current_request_id);

      m_current_provider_id++;
    }
}


void CoverTask::slot_cover_provider_search_finished()
{
    //Debug::debug() << "       [CoverTask] slot_cover_provider_search_finished  m_current_request_id:" << m_current_request_id;

    if(m_current_provider_id < m_coverprovider_list.size() )
    {
      slot_start_cover_provider(); 
    }
    else
    {
      m_current_request_id++;
      slot_process_cover_search();
    }
}

//! -- slot_save_cover ---------------------------------------------------------
void CoverTask::slot_save_cover(int request_id, QByteArray bytes)
{
    Debug::debug() << "       [CoverTask] slot_save_cover";
    
    if(m_current_request_id != request_id) {
      Debug::warning() << "       [CoverTask] bad request id";
      //! process next request
      m_current_request_id++;
      slot_process_cover_search();      
      return;
    }

    //! filepath for cover file
    const QString filePath = UTIL::CONFIGDIR + "/albums/" + m_current_param.cover_path;

    QFile file(filePath);
    if(file.exists())
    {
      Debug::debug() << "       [CoverTask] file exists, start next cover search";
    }
    else if (!file.open(QIODevice::WriteOnly))
    {
      Debug::debug() << "       [CoverTask] error writing album image";
    }
    else
    {
      QImage image = QImage::fromData(bytes);
      image = image.scaled(QSize(110, 110), Qt::KeepAspectRatio, Qt::SmoothTransformation);
      image.save(filePath, "png", -1);
    }

    //! process next request
    m_current_request_id++;
    slot_process_cover_search();
}


//! -- slot_check_database -----------------------------------------------------
void CoverTask::slot_check_database()
{
    //Debug::debug() << "       [CoverThread] slot_check_database";

    Database db;
    if (!db.connect())  return;

    m_current_request_id = 1;
    m_map_params.clear();
    
    //! albums database loop
    QSqlQuery query("SELECT name,cover,artist_name FROM view_albums",*db.sqlDb());
    while (query.next())
    {
        RequestParam param = RequestParam();

        param.artist     = query.value(2).toString();  // artist
        param.album      = query.value(0).toString();  // album
        param.cover_path = query.value(1).toString();  // covername

        const QString path = UTIL::CONFIGDIR + "/albums/" + param.cover_path;

        if(!QFile::exists(path)) {
          //m_list_param.append (param);
          m_map_params[m_current_request_id++] = param;
        }
    } // fin while album query
    
    m_current_request_id = 1;
}


