/****************************************************************************************
*  YAROCK                                                                               *
*  Copyright (c) 2010-2013 Sebastien amardeilh <sebastien.amardeilh+yarock@gmail.com>   *
*                                                                                       *
*  This program is free software; you can redistribute it and/or modify it under        *
*  the terms of the GNU General Public License as published by the Free Software        *
*  Foundation; either version 2 of the License, or (at your option) any later           *
*  version.                                                                             *
*                                                                                       *
*  This program is distributed in the hope that it will be useful, but WITHOUT ANY      *
*  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A      *
*  PARTICULAR PURPOSE. See the GNU General Public License for more details.             *
*                                                                                       *
*  You should have received a copy of the GNU General Public License along with         *
*  this program.  If not, see <http://www.gnu.org/licenses/>.                           *
*****************************************************************************************/
#include "graphicitem_menu.h"

#include "graphicitem.h"
#include "collectionmodel.h"
#include "collectionscene.h"

#include "plsfilemodel.h"
#include "streammodel.h"

#include "mediaitem.h"
#include "global_actions.h"
#include "yarocksettings.h"

#include "debug.h"
/*
********************************************************************************
*                                                                              *
*    Class MenuTrack                                                           *
*                                                                              *
********************************************************************************
*/
class MenuTrack : public QWidget
{
public:
    MenuTrack(QWidget* parent,const QString& text) : QWidget(parent)
    {
       setSizePolicy(QSizePolicy::Expanding, QSizePolicy::MinimumExpanding);
       m_text     = text ;
       m_parent   = parent;
       mousehover = false;
    }

protected:
    void paintEvent(QPaintEvent *event);
    QSize sizeHint() const;

private:
    QString  m_text;
    QWidget *m_parent;
    bool     mousehover;
};

QSize MenuTrack::sizeHint() const
{
    QFontMetrics fm( QApplication::font() );
    const int x = fm.boundingRect(m_text).width() + 12 /* for icon */ + 20;
    const int y = fm.height();
    return QSize(x, y);
}


void MenuTrack::paintEvent(QPaintEvent *event)
{
    QPainter p(this);
    p.setFont( QApplication::font() );


    QRect rect = event->rect();
    if(mousehover) {
      p.setPen(Qt::NoPen);
      QColor brush_color = QApplication::palette().color(QPalette::Normal, QPalette::Highlight);
      p.setBrush(QBrush( brush_color ,Qt::SolidPattern));
      p.drawRect(rect);
      p.setPen(QApplication::palette().color(QPalette::Normal, QPalette::HighlightedText));
    }
    else {
      p.setPen(QApplication::palette().color(QPalette::Normal, QPalette::WindowText));
    }

    QIcon icon = QIcon(":/images/track-48x48.png");
    icon.paint(&p, 10, rect.top()+2, 12, 12, Qt::AlignVCenter | Qt::AlignLeft, QIcon::Normal);
    p.drawText(25,rect.top(),rect.width(),rect.height(),Qt::AlignVCenter | Qt::AlignLeft, m_text);

    p.end();
}

/*
********************************************************************************
*                                                                              *
*    Class HeaderWidget                                                        *
*                                                                              *
********************************************************************************
*/
class HeaderWidget : public QWidget
{
public:
    HeaderWidget(QWidget* parent) : QWidget(parent)
    {
      QPalette palette = QApplication::palette();
      palette.setColor(QPalette::Window , SETTINGS()->_baseColor);
      palette.setColor(QPalette::WindowText, QApplication::palette().color(QPalette::Normal, QPalette::BrightText));
      
      this->setAutoFillBackground(true);
      this->setPalette(palette);
      this->setMinimumHeight(35);

      ui_label = new QLabel(this);
      ui_label->setFont(QFont("Arial",10,QFont::Bold));
      ui_label->setPalette(palette);

      ui_rating = new RatingWidget(this);
      ui_rating->setMinimumHeight(22);
      ui_rating->set_draw_frame(false);

      QHBoxLayout *hbl1 = new QHBoxLayout();
      hbl1->setContentsMargins(0,0,0,0);
      hbl1->setSpacing(0);
      hbl1->setMargin(0);

      hbl1->addItem(new QSpacerItem(20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum));
      hbl1->addWidget(ui_label);
      hbl1->addItem(new QSpacerItem(20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum));

      QVBoxLayout* layout = new QVBoxLayout(this);
      layout->setContentsMargins(0,0,0,0);
      layout->setSpacing(0);
      layout->setMargin(0);
      layout->addLayout(hbl1);
      layout->addWidget(ui_rating);

      this->setLayout(layout);
    }

    void setItems(QList<QGraphicsItem*> items, bool isSelection=false);

    RatingWidget* ratingWidget() {return ui_rating;}
     
    void apply_rating(float rating);
     
private:
    void load_rating();

private:
    bool                      m_isSelection;
    QList<MEDIA::MediaPtr>    m_media_list;

    QLabel                   *ui_label;
    RatingWidget             *ui_rating;
};



void HeaderWidget::setItems(QList<QGraphicsItem*> items, bool isSelection)
{
    //Debug::debug() << "HeaderWidget::setItems";
    m_isSelection = isSelection;

    // get media items from graphics item
    m_media_list.clear();
    foreach(QGraphicsItem* gi, items) 
    {
      switch(gi->type()) {
        case GraphicsItem::AlbumType  : m_media_list << (static_cast<AlbumGraphicItem*>(gi))->media;  break;
	case GraphicsItem::AlbumGenreType : m_media_list << (static_cast<AlbumGenreGraphicItem*>(gi))->media;  break;
        case GraphicsItem::ArtistType : m_media_list << (static_cast<ArtistGraphicItem*>(gi))->media; break;
        case GraphicsItem::TrackType  : m_media_list << (static_cast<TrackGraphicItem*>(gi))->media;  break;
        case GraphicsItem::StreamType : m_media_list << (static_cast<StreamGraphicItem*>(gi))->media; break;
        case GraphicsItem::PlaylistType: m_media_list << (static_cast<PlaylistGraphicItem*>(gi))->media;break;
        default : Debug::warning() << "HeaderWidget : unknow graphic item";
      }
    }
    
    // build text label
    QString text;
    
    if(!m_isSelection) {
      switch (m_media_list.first()->type())
      {
        case TYPE_ARTIST   : text = MEDIA::ArtistPtr::staticCast(m_media_list.first())->name;break;
        case TYPE_ALBUM    : text = MEDIA::AlbumPtr::staticCast(m_media_list.first())->name;break;
        case TYPE_TRACK    : text = MEDIA::TrackPtr::staticCast(m_media_list.first())->title;break;
        case TYPE_STREAM   : text = MEDIA::TrackPtr::staticCast(m_media_list.first())->url;break;
        case TYPE_PLAYLIST : text = MEDIA::PlaylistPtr::staticCast(m_media_list.first())->name;break;
        default: break;
      }
    }
    else {
      switch(m_media_list.first()->type()) {
        case TYPE_ARTIST     : text = QString(tr("%1 artists")).arg(m_media_list.count());break;
        case TYPE_ALBUM      : text = QString(tr("%1 albums")).arg(m_media_list.count());break;
        case TYPE_TRACK      : text = QString(tr("%1 tracks")).arg(m_media_list.count());break;
        case TYPE_STREAM     : text = QString(tr("%1 streams")).arg(m_media_list.count());break;
        case TYPE_PLAYLIST   : text = QString(tr("%1 playlist")).arg(m_media_list.count());break;
        default: break;
      }    
    }

    QFontMetrics metric( QFont("Arial",10,QFont::Bold) );
    const int x = metric.boundingRect(text).width();
    if(x > 200)
      text = metric.elidedText ( text, Qt::ElideMiddle, 200);

    ui_label->setText(text);

    // load rating from media
    load_rating();

    this->update();
}

// private
void HeaderWidget::load_rating()
{
    //Debug::debug() << "HeaderWidget::load_rating";
    bool isUser = true;

    if(m_isSelection)
    {
      ui_rating->set_rating( 0 );
    }
    else
    {
      MEDIA::MediaPtr media = m_media_list.first();

      if(!media)
      {
        ui_rating->hide();
        return;
      }
      else if(media->type() == TYPE_ARTIST)
      {
        MEDIA::ArtistPtr artist = MEDIA::ArtistPtr::staticCast(media);
        isUser = artist->isUserRating;
        if(isUser)
           ui_rating->set_rating( artist->rating );
        else
           ui_rating->set_rating( CollectionModel::instance()->getItemAutoRating(media)  );
      }
      else if (media->type() == TYPE_ALBUM)
      {
        MEDIA::AlbumPtr album = MEDIA::AlbumPtr::staticCast(media);
        isUser = album->isUserRating;
        if(isUser)
          ui_rating->set_rating( album->rating );
        else
          ui_rating->set_rating( CollectionModel::instance()->getItemAutoRating(media));
      }
      else if (media->type() == TYPE_TRACK)
      {
        MEDIA::TrackPtr track = MEDIA::TrackPtr::staticCast(media);
        ui_rating->set_rating( track->rating );
      }
      else
      {
        ui_rating->hide();
        return;
      }
    }

    ui_rating->set_user_rating(isUser);
    ui_rating->update();
}

void HeaderWidget::apply_rating(float rating)
{
    //Debug::debug() << " HeaderWidget::apply_rating : " << rating;
    foreach(MEDIA::MediaPtr media, m_media_list)
    {
      switch (media->type()) {
        case TYPE_ARTIST :
        {
           MEDIA::ArtistPtr artist = MEDIA::ArtistPtr::staticCast(media);
           artist->rating          = rating;
           artist->isUserRating    = true;
           break;
        }
        case TYPE_ALBUM  :
        {
           MEDIA::AlbumPtr album = MEDIA::AlbumPtr::staticCast(media);
           album->rating         = rating;
           album->isUserRating   = true;
           break;
        }
        case TYPE_TRACK  :
        {
           MEDIA::TrackPtr track = MEDIA::TrackPtr::staticCast(media);
           track->rating         =  rating;
           break;
        }
        default: break;
      }
    }

    ui_rating->set_user_rating(true);
    ui_rating->update();
}


/*
********************************************************************************
*                                                                              *
*    Class GraphicsItemMenu                                                    *
*                                                                              *
********************************************************************************
*/
GraphicsItemMenu::GraphicsItemMenu(QWidget *parent) : QMenu(parent)
{
    this->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Minimum);
    this->setAutoFillBackground(true);
    this->setContentsMargins(0,0,0,0);
    this->setWindowFlags( Qt::Popup | Qt::FramelessWindowHint );
    
    QPalette palette = QApplication::palette();
    palette.setColor(QPalette::Background, palette.color(QPalette::Base));
    this->setPalette(palette);
 
    //! creates menu actions
    m_map_actions.insert(ALBUM_PLAY,          new QAction(QIcon(":/images/media-play.png"), tr("Play"), 0));
    m_map_actions.insert(ALBUM_LOAD_COVER,    new QAction(QIcon(":/images/album.png"), tr("Load cover art from file"), 0));
    m_map_actions.insert(ALBUM_LASTFM_COVER,  new QAction(QIcon(":/images/download-48x48.png"), tr("Download cover art"), 0));
    m_map_actions.insert(ALBUM_REMOVE_COVER,  new QAction(QIcon::fromTheme("edit-delete"), tr("Remove cover art"), 0));
    m_map_actions.insert(ALBUM_EDIT,          new QAction(QIcon(":/images/edit_info.png"), tr("Edit"), 0));
    m_map_actions.insert(ALBUM_QUEUE_END,     new QAction(QIcon(":/images/playqueue_enqueue_end-48x48.png"), tr("Enqueue to playqueue"), 0));
    m_map_actions.insert(ALBUM_QUEUE_NEW,     new QAction(QIcon(":/images/playqueue_enqueue_new-48x48.png"), tr("Enqueue as new playqueue"), 0));
    m_map_actions.insert(ALBUM_FAVORITE,      new QAction(QIcon(":/images/favorites-48x48.png"), tr("Add to favorites"), 0));

    m_map_actions.insert(GENRE_PLAY,          new QAction(QIcon(":/images/media-play.png"), tr("Play"), 0));
    m_map_actions.insert(GENRE_QUEUE_END,     new QAction(QIcon(":/images/playqueue_enqueue_end-48x48.png"), tr("Enqueue to playqueue"), 0));
    m_map_actions.insert(GENRE_QUEUE_NEW,     new QAction(QIcon(":/images/playqueue_enqueue_new-48x48.png"), tr("Enqueue as new playqueue"), 0));
    m_map_actions.insert(GENRE_EDIT,          new QAction(QIcon(":/images/edit_info.png"), tr("Edit"), 0));

    m_map_actions.insert(ARTIST_PLAY,         new QAction(QIcon(":/images/media-play.png"), tr("Play"), 0));
    m_map_actions.insert(ARTIST_EDIT,         new QAction(QIcon(":/images/edit_info.png"), tr("Edit"), 0));
    m_map_actions.insert(ARTIST_QUEUE_END,    new QAction(QIcon(":/images/playqueue_enqueue_end-48x48.png"), tr("Enqueue to playqueue"), 0));
    m_map_actions.insert(ARTIST_QUEUE_NEW,    new QAction(QIcon(":/images/playqueue_enqueue_new-48x48.png"), tr("Enqueue as new playqueue"), 0));
    m_map_actions.insert(ARTIST_FAVORITE,     new QAction(QIcon(":/images/favorites-48x48.png"), tr("Add to favorites"), 0));

    m_map_actions.insert(PLAYLIST_PLAY,       new QAction(QIcon(":/images/media-play.png"), tr("Play"), 0));
    m_map_actions.insert(PLAYLIST_EDIT,       new QAction(QIcon(":/images/edit_info.png"), tr("Edit"), 0));
    m_map_actions.insert(PLAYLIST_QUEUE_END,  new QAction(QIcon(":/images/playqueue_enqueue_end-48x48.png"), tr("Enqueue to playqueue"), 0));
    m_map_actions.insert(PLAYLIST_QUEUE_NEW,  new QAction(QIcon(":/images/playqueue_enqueue_new-48x48.png"), tr("Enqueue as new playqueue"), 0));
    m_map_actions.insert(PLAYLIST_FAVORITE,   new QAction(QIcon(":/images/favorites-48x48.png"), tr("Add to favorites"), 0));
    m_map_actions.insert(PLAYLIST_REMOVE,     new QAction(QIcon::fromTheme("edit-delete"),tr("&Remove playlist from disk"), 0));

    m_map_actions.insert(TRACK_PLAY,          new QAction(QIcon(":/images/media-play.png"), tr("Play"), 0));
    m_map_actions.insert(TRACK_EDIT,          new QAction(QIcon(":/images/edit_info.png"), tr("Edit"), 0));
    m_map_actions.insert(TRACK_QUEUE_END,     new QAction(QIcon(":/images/playqueue_enqueue_end-48x48.png"), tr("Enqueue to playqueue"), 0));
    m_map_actions.insert(TRACK_QUEUE_NEW,     new QAction(QIcon(":/images/playqueue_enqueue_new-48x48.png"), tr("Enqueue as new playqueue"), 0));

    m_map_actions.insert(STREAM_PLAY,          new QAction(QIcon(":/images/media-play.png"), tr("Play"), 0));
    m_map_actions.insert(STREAM_EDIT,          new QAction(QIcon(":/images/edit_info.png"), tr("Edit"), 0));
    m_map_actions.insert(STREAM_QUEUE_END,     new QAction(QIcon(":/images/playqueue_enqueue_end-48x48.png"), tr("Enqueue to playqueue"), 0));
    m_map_actions.insert(STREAM_QUEUE_NEW,     new QAction(QIcon(":/images/playqueue_enqueue_new-48x48.png"), tr("Enqueue as new playqueue"), 0));
    m_map_actions.insert(STREAM_FAVORITE,      new QAction(QIcon(":/images/favorites-48x48.png"), tr("Add to favorites"), 0));

    m_map_actions.insert(SELECTION_PLAY,       new QAction(QIcon(":/images/media-play.png"), tr("Play"), 0));
    m_map_actions.insert(SELECTION_QUEUE_END,  new QAction(QIcon(":/images/playqueue_enqueue_end-48x48.png"), tr("Enqueue to playqueue"), 0));
    m_map_actions.insert(SELECTION_QUEUE_NEW,  new QAction(QIcon(":/images/playqueue_enqueue_new-48x48.png"), tr("Enqueue as new playqueue"), 0));


    // connect each actions
    foreach(QAction * action, m_map_actions.values() ) {
      QObject::connect(action, SIGNAL(triggered()), this, SLOT(slot_actionsTriggered()));
    }
}

// public
void GraphicsItemMenu::appendItem(QGraphicsItem* item)
{
    m_items.clear();
    m_items << item;
}

void GraphicsItemMenu::appendItems(QList<QGraphicsItem*> items)
{
    m_items.clear();
    m_items << items;
}

void GraphicsItemMenu::updateMenu(bool isSelection)
{
  //Debug::debug() << "GraphicsItemMenu::updateMenu";
  
  // delete action_widget
  this->clear();
  
  is_selection = isSelection;
  
  if(m_items.isEmpty())
    return;

  /*-------------------------------------------------*/
  /* Selected items                                  */
  /* ------------------------------------------------*/
  if( is_selection )
  {
    //Debug::debug() << "GraphicsItemMenu::Selected items";

    QWidget* main_widget = new QWidget();

    // header widget
    m_header = new HeaderWidget(main_widget);
    m_header->setItems( m_items, is_selection);
    connect(m_header->ratingWidget(), SIGNAL(RatingChanged(float)), this, SLOT(slot_rating_changed(float)));

    // toolbar
    QToolBar *tool_bar = new QToolBar(main_widget);
    tool_bar->setIconSize( QSize( 16, 16 ) );
    tool_bar->addAction( m_map_actions.value(SELECTION_PLAY) );
    tool_bar->addAction( m_map_actions.value(SELECTION_QUEUE_END) );
    tool_bar->addAction( m_map_actions.value(SELECTION_QUEUE_NEW) );

    QHBoxLayout * hbox = new QHBoxLayout();
    hbox->addItem(new QSpacerItem(20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum));
    hbox->addWidget(tool_bar);
    hbox->addItem(new QSpacerItem(20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum));

    // main widget layout
    QVBoxLayout * main_box = new QVBoxLayout();
    main_box->setSpacing(0);
    main_box->setContentsMargins(0,0,0,0);
    main_box->addWidget(m_header);
    main_box->addLayout(hbox);
    main_widget->setLayout(main_box);

    QWidgetAction* action_widget = new QWidgetAction(this);
    action_widget->setDefaultWidget( main_widget );
    this->addAction( action_widget );

    return;
  }



  /*-------------------------------------------------*/
  /* AlbumType                                       */
  /* ------------------------------------------------*/
  if( m_items.first()->type() == GraphicsItem::AlbumType )
  {
    AlbumGraphicItem *item = static_cast<AlbumGraphicItem*>(m_items.first());

    QWidget* main_widget = new QWidget();

    m_header = new HeaderWidget(main_widget);
    m_header->setItems(m_items, is_selection);
    connect(m_header->ratingWidget(), SIGNAL(RatingChanged(float)), this, SLOT(slot_rating_changed(float)));


    QToolBar *tool_bar = new QToolBar(main_widget);
    tool_bar->setIconSize( QSize( 16, 16 ) );
    tool_bar->addAction( m_map_actions.value(ALBUM_PLAY) );
    tool_bar->addAction( m_map_actions.value(ALBUM_QUEUE_END) );
    tool_bar->addAction( m_map_actions.value(ALBUM_QUEUE_NEW) );
    tool_bar->addSeparator();
    tool_bar->addAction( m_map_actions.value(ALBUM_LOAD_COVER) );
    tool_bar->addAction( m_map_actions.value(ALBUM_LASTFM_COVER) );
    tool_bar->addAction( m_map_actions.value(ALBUM_REMOVE_COVER) );
    tool_bar->addAction( m_map_actions.value(ALBUM_EDIT) );
    tool_bar->addSeparator();
    tool_bar->addAction( m_map_actions.value(ALBUM_FAVORITE) );

    QHBoxLayout * hbox = new QHBoxLayout();
    hbox->addItem(new QSpacerItem(20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum));
    hbox->addWidget(tool_bar);
    hbox->addItem(new QSpacerItem(20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum));

    QVBoxLayout *vbox2 = new QVBoxLayout();
    vbox2->setContentsMargins(0,4,0,4);
    vbox2->setSpacing(1);

    // update data from album media item
    QList<MEDIA::TrackPtr> list = CollectionModel::instance()->getItemChildrenTracks(item->media);
    foreach(MEDIA::TrackPtr media, list) {
        vbox2->addWidget(new MenuTrack(0,media->title));
    }


    if(item->media->isFavorite)
      m_map_actions.value(ALBUM_FAVORITE)->setText( tr("Remove from favorites") );
    else
      m_map_actions.value(ALBUM_FAVORITE)->setText( tr("Add to favorites") );

    QVBoxLayout * main_box = new QVBoxLayout();
    main_box->setSpacing(0);
    main_box->setContentsMargins(0,0,0,0);
    main_box->addWidget(m_header);
    main_box->addLayout(vbox2);
    main_box->addLayout(hbox);
    main_widget->setLayout(main_box);

    QWidgetAction* action_widget = new QWidgetAction(this);
    action_widget->setDefaultWidget( main_widget );
    this->addAction( action_widget );
  }
  /*-------------------------------------------------*/
  /* AlbumGenreType                                  */
  /* ------------------------------------------------*/
  else if( m_items.first()->type() == GraphicsItem::AlbumGenreType )
  {
    AlbumGenreGraphicItem *item = static_cast<AlbumGenreGraphicItem*>(m_items.first());

    QWidget* main_widget = new QWidget();

    m_header = new HeaderWidget(main_widget);
    m_header->setItems(m_items, is_selection );
    connect(m_header->ratingWidget(), SIGNAL(RatingChanged(float)), this, SLOT(slot_rating_changed(float)));

    QToolBar *tool_bar = new QToolBar(main_widget);
    tool_bar->setIconSize( QSize( 16, 16 ) );
    tool_bar->addAction( m_map_actions.value(GENRE_PLAY) );
    tool_bar->addAction( m_map_actions.value(GENRE_QUEUE_END) );
    tool_bar->addAction( m_map_actions.value(GENRE_QUEUE_NEW) );
    tool_bar->addSeparator();
    tool_bar->addAction( m_map_actions.value(ALBUM_LOAD_COVER) );
    tool_bar->addAction( m_map_actions.value(ALBUM_LASTFM_COVER) );
    tool_bar->addAction( m_map_actions.value(ALBUM_REMOVE_COVER) );
    tool_bar->addAction( m_map_actions.value(GENRE_EDIT) );

    QHBoxLayout * hbox = new QHBoxLayout();
    hbox->addItem(new QSpacerItem(20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum));
    hbox->addWidget(tool_bar);
    hbox->addItem(new QSpacerItem(20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum));

    QVBoxLayout *vbox2 = new QVBoxLayout();
    vbox2->setContentsMargins(0,4,0,4);

    QList<MEDIA::TrackPtr> list = CollectionModel::instance()->getAlbumChildrenTracksGenre(item->media,item->_genre);
    foreach(MEDIA::TrackPtr media, list) {
        vbox2->addWidget(new MenuTrack(0,media->title) );
    }


    QVBoxLayout * main_box = new QVBoxLayout();
    main_box->setSpacing(0);
    main_box->setContentsMargins(0,0,0,0);
    main_box->addWidget(m_header);
    main_box->addLayout(vbox2);
    main_box->addLayout(hbox);
    main_widget->setLayout(main_box);

    QWidgetAction * action_widget = new QWidgetAction(this);
    action_widget->setDefaultWidget( main_widget );
    this->addAction( action_widget );
  }
  /*-------------------------------------------------*/
  /* ArtistType                                      */
  /* ------------------------------------------------*/
  else if( m_items.first()->type() == GraphicsItem::ArtistType )
  {
    //Debug::debug() << "GraphicsItemMenu::ArtistType";
    ArtistGraphicItem *item = static_cast<ArtistGraphicItem*>(m_items.first());

    QWidget* main_widget = new QWidget();

    m_header = new HeaderWidget(main_widget);
    m_header->setItems(m_items, is_selection );
    connect(m_header->ratingWidget(), SIGNAL(RatingChanged(float)), this, SLOT(slot_rating_changed(float)));

    QToolBar *tool_bar = new QToolBar(main_widget);
    tool_bar->setIconSize( QSize( 16, 16 ) );
    tool_bar->addAction( m_map_actions.value(ARTIST_PLAY) );
    tool_bar->addAction( m_map_actions.value(ARTIST_QUEUE_END) );
    tool_bar->addAction( m_map_actions.value(ARTIST_QUEUE_NEW) );
    tool_bar->addSeparator();
    tool_bar->addAction( m_map_actions.value(ARTIST_EDIT) );
    tool_bar->addSeparator();
    tool_bar->addAction( m_map_actions.value(ARTIST_FAVORITE) );

    QHBoxLayout * hbox = new QHBoxLayout();
    hbox->addItem(new QSpacerItem(20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum));
    hbox->addWidget(tool_bar);
    hbox->addItem(new QSpacerItem(20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum));


    if(item->media->isFavorite)
      m_map_actions.value(ARTIST_FAVORITE)->setText( tr("Remove from favorites") );
    else
      m_map_actions.value(ARTIST_FAVORITE)->setText( tr("Add to favorites") );

    QVBoxLayout * main_box = new QVBoxLayout();
    main_box->setSpacing(0);
    main_box->setContentsMargins(0,0,0,0);
    main_box->addWidget(m_header);
    main_box->addLayout(hbox);
    main_widget->setLayout(main_box);

    QWidgetAction * action_widget = new QWidgetAction(this);
    action_widget->setDefaultWidget( main_widget );
    this->addAction( action_widget );
  }
  /*-------------------------------------------------*/
  /* TrackType                                       */
  /* ------------------------------------------------*/
  else if( m_items.first()->type() == GraphicsItem::TrackType )
  {
    TrackGraphicItem *item = static_cast<TrackGraphicItem*>(m_items.first());

    QWidget* main_widget = new QWidget();

    m_header = new HeaderWidget(main_widget);
    m_header->setItems(m_items,is_selection);
    connect(m_header->ratingWidget(), SIGNAL(RatingChanged(float)), this, SLOT(slot_rating_changed(float)));

    QToolBar *tool_bar = new QToolBar(main_widget);
    tool_bar->setIconSize( QSize( 16, 16 ) );

    tool_bar->addAction( m_map_actions.value(TRACK_PLAY) );

    if(item->media->id != -1)
      tool_bar->addAction( m_map_actions.value(TRACK_EDIT) );

    tool_bar->addAction( m_map_actions.value(TRACK_QUEUE_END) );
    tool_bar->addAction( m_map_actions.value(TRACK_QUEUE_NEW) );

    QHBoxLayout * hbox = new QHBoxLayout();
    hbox->addItem(new QSpacerItem(20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum));
    hbox->addWidget(tool_bar);
    hbox->addItem(new QSpacerItem(20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum));

    QVBoxLayout * main_box = new QVBoxLayout();
    main_box->setSpacing(0);
    main_box->setContentsMargins(0,0,0,0);
    main_box->addWidget(m_header);
    main_box->addLayout(hbox);
    main_widget->setLayout(main_box);

    QWidgetAction * action_widget = new QWidgetAction(this);
    action_widget->setDefaultWidget( main_widget );
    this->addAction( action_widget );
  }
  /*-------------------------------------------------*/
  /* PlaylistType                                    */
  /* ------------------------------------------------*/
  else if( m_items.first()->type() == GraphicsItem::PlaylistType )
  {
    PlaylistGraphicItem *item = static_cast<PlaylistGraphicItem*>(m_items.first());

    QWidget* main_widget = new QWidget();

    m_header = new HeaderWidget(main_widget);
    m_header->setItems(m_items,is_selection);
    connect(m_header->ratingWidget(), SIGNAL(RatingChanged(float)), this, SLOT(slot_rating_changed(float)));

    QToolBar *tool_bar = new QToolBar(main_widget);
    tool_bar->setIconSize( QSize( 16, 16 ) );
    tool_bar->addAction( m_map_actions.value(PLAYLIST_PLAY) );
    tool_bar->addAction( m_map_actions.value(PLAYLIST_QUEUE_END) );
    tool_bar->addAction( m_map_actions.value(PLAYLIST_QUEUE_NEW) );
    tool_bar->addSeparator();
    tool_bar->addAction( m_map_actions.value(PLAYLIST_REMOVE) );
    tool_bar->addAction( m_map_actions.value(PLAYLIST_EDIT) );
    tool_bar->addAction( m_map_actions.value(PLAYLIST_FAVORITE) );

    if(item->media->isFavorite)
      m_map_actions.value(PLAYLIST_FAVORITE)->setText( tr("Remove from favorites") );
    else
      m_map_actions.value(PLAYLIST_FAVORITE)->setText( tr("Add to favorites") );

    QHBoxLayout * hbox = new QHBoxLayout();
    hbox->addItem(new QSpacerItem(20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum));
    hbox->addWidget(tool_bar);
    hbox->addItem(new QSpacerItem(20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum));

    QVBoxLayout * main_box = new QVBoxLayout();
    main_box->setSpacing(0);
    main_box->setContentsMargins(0,0,0,0);
    main_box->addWidget(m_header);
    main_box->addLayout(hbox);
    main_widget->setLayout(main_box);

    QWidgetAction * action_widget = new QWidgetAction(this);
    action_widget->setDefaultWidget( main_widget );
    this->addAction( action_widget );
  }
  /*-------------------------------------------------*/
  /* StreamType                                      */
  /* ------------------------------------------------*/
  else if( m_items.first()->type() == GraphicsItem::StreamType )
  {
    StreamGraphicItem *item = static_cast<StreamGraphicItem*>(m_items.first());

    QWidget* main_widget = new QWidget();

    m_header = new HeaderWidget(main_widget);
    m_header->setItems(m_items,is_selection);
    connect(m_header->ratingWidget(), SIGNAL(RatingChanged(float)), this, SLOT(slot_rating_changed(float)));

    QToolBar *tool_bar = new QToolBar(main_widget);
    tool_bar->setIconSize( QSize( 16, 16 ) );
    tool_bar->addAction( m_map_actions.value(STREAM_PLAY) );
    tool_bar->addAction( m_map_actions.value(STREAM_QUEUE_END) );
    tool_bar->addAction( m_map_actions.value(STREAM_QUEUE_NEW) );
    tool_bar->addAction( m_map_actions.value(STREAM_FAVORITE) );

    if(item->media->isFavorite)
      m_map_actions.value(STREAM_FAVORITE)->setText( tr("Remove from favorites") );
    else
      m_map_actions.value(STREAM_FAVORITE)->setText( tr("Add to favorites") );

    QHBoxLayout * hbox = new QHBoxLayout();
    hbox->addItem(new QSpacerItem(20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum));
    hbox->addWidget(tool_bar);
    hbox->addItem(new QSpacerItem(20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum));

    QVBoxLayout * main_box = new QVBoxLayout();
    main_box->setSpacing(0);
    main_box->setContentsMargins(0,0,0,0);
    main_box->addWidget(m_header);
    main_box->addLayout(hbox);
    main_widget->setLayout(main_box);

    QWidgetAction * action_widget = new QWidgetAction(this);
    action_widget->setDefaultWidget( main_widget );
    this->addAction( action_widget );
  }
}



/*******************************************************************************
    slot_actionsTriggered  (PRIVATE SLOTS)
*******************************************************************************/
void GraphicsItemMenu::slot_actionsTriggered()
{
    this->hide();
    QAction *action = qobject_cast<QAction *>(sender());
    if(action)
      emit menu_action_triggered( (ENUM_ACTION_ITEM_MENU)m_map_actions.key(action) );
}

/*******************************************************************************
    slot_rating_changed  (PRIVATE SLOTS)
*******************************************************************************/
void GraphicsItemMenu::slot_rating_changed(float rating)
{
    Debug::debug() << " GraphicsItemMenu::slot_rating_changed";
    m_header->apply_rating(rating);

    if(is_selection)
    {
      m_scene->rateSelection(m_items);
    }
    else
    {
      QVariant v;
      v.setValue(static_cast<QGraphicsItem*>(m_items.first()));

      (ACTIONS()->value(BROWSER_ITEM_RATING_CLICK))->setData(v);
      (ACTIONS()->value(BROWSER_ITEM_RATING_CLICK))->trigger();
    }
}
